/* Unicorn Engine */
/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2015-2017 */
/* This file is released under LGPL2.
   See COPYING.LGPL2 in root directory for more details
*/

#ifndef QNICORN_ARM_H
#define QNICORN_ARM_H

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _MSC_VER
#pragma warning(disable : 4201)
#endif

//> ARM CPU
typedef enum qc_cpu_arm {
    QC_CPU_ARM_926 = 0,
    QC_CPU_ARM_946,
    QC_CPU_ARM_1026,
    QC_CPU_ARM_1136_R2,
    QC_CPU_ARM_1136,
    QC_CPU_ARM_1176,
    QC_CPU_ARM_11MPCORE,
    QC_CPU_ARM_CORTEX_M0,
    QC_CPU_ARM_CORTEX_M3,
    QC_CPU_ARM_CORTEX_M4,
    QC_CPU_ARM_CORTEX_M7,
    QC_CPU_ARM_CORTEX_M33,
    QC_CPU_ARM_CORTEX_R5,
    QC_CPU_ARM_CORTEX_R5F,
    QC_CPU_ARM_CORTEX_A7,
    QC_CPU_ARM_CORTEX_A8,
    QC_CPU_ARM_CORTEX_A9,
    QC_CPU_ARM_CORTEX_A15,
    QC_CPU_ARM_TI925T,
    QC_CPU_ARM_SA1100,
    QC_CPU_ARM_SA1110,
    QC_CPU_ARM_PXA250,
    QC_CPU_ARM_PXA255,
    QC_CPU_ARM_PXA260,
    QC_CPU_ARM_PXA261,
    QC_CPU_ARM_PXA262,
    QC_CPU_ARM_PXA270A0,
    QC_CPU_ARM_PXA270A1,
    QC_CPU_ARM_PXA270B0,
    QC_CPU_ARM_PXA270B1,
    QC_CPU_ARM_PXA270C0,
    QC_CPU_ARM_PXA270C5,
    QC_CPU_ARM_MAX
} qc_cpu_arm;

//> ARM registers
typedef enum qc_arm_reg {
    QC_ARM_REG_INVALID = 0,
    QC_ARM_REG_APSR,
    QC_ARM_REG_APSR_NZCV,
    QC_ARM_REG_CPSR,
    QC_ARM_REG_FPEXC,
    QC_ARM_REG_FPINST,
    QC_ARM_REG_FPSCR,
    QC_ARM_REG_FPSCR_NZCV,
    QC_ARM_REG_FPSID,
    QC_ARM_REG_ITSTATE,
    QC_ARM_REG_LR,
    QC_ARM_REG_PC,
    QC_ARM_REG_SP,
    QC_ARM_REG_SPSR,
    QC_ARM_REG_D0,
    QC_ARM_REG_D1,
    QC_ARM_REG_D2,
    QC_ARM_REG_D3,
    QC_ARM_REG_D4,
    QC_ARM_REG_D5,
    QC_ARM_REG_D6,
    QC_ARM_REG_D7,
    QC_ARM_REG_D8,
    QC_ARM_REG_D9,
    QC_ARM_REG_D10,
    QC_ARM_REG_D11,
    QC_ARM_REG_D12,
    QC_ARM_REG_D13,
    QC_ARM_REG_D14,
    QC_ARM_REG_D15,
    QC_ARM_REG_D16,
    QC_ARM_REG_D17,
    QC_ARM_REG_D18,
    QC_ARM_REG_D19,
    QC_ARM_REG_D20,
    QC_ARM_REG_D21,
    QC_ARM_REG_D22,
    QC_ARM_REG_D23,
    QC_ARM_REG_D24,
    QC_ARM_REG_D25,
    QC_ARM_REG_D26,
    QC_ARM_REG_D27,
    QC_ARM_REG_D28,
    QC_ARM_REG_D29,
    QC_ARM_REG_D30,
    QC_ARM_REG_D31,
    QC_ARM_REG_FPINST2,
    QC_ARM_REG_MVFR0,
    QC_ARM_REG_MVFR1,
    QC_ARM_REG_MVFR2,
    QC_ARM_REG_Q0,
    QC_ARM_REG_Q1,
    QC_ARM_REG_Q2,
    QC_ARM_REG_Q3,
    QC_ARM_REG_Q4,
    QC_ARM_REG_Q5,
    QC_ARM_REG_Q6,
    QC_ARM_REG_Q7,
    QC_ARM_REG_Q8,
    QC_ARM_REG_Q9,
    QC_ARM_REG_Q10,
    QC_ARM_REG_Q11,
    QC_ARM_REG_Q12,
    QC_ARM_REG_Q13,
    QC_ARM_REG_Q14,
    QC_ARM_REG_Q15,
    QC_ARM_REG_R0,
    QC_ARM_REG_R1,
    QC_ARM_REG_R2,
    QC_ARM_REG_R3,
    QC_ARM_REG_R4,
    QC_ARM_REG_R5,
    QC_ARM_REG_R6,
    QC_ARM_REG_R7,
    QC_ARM_REG_R8,
    QC_ARM_REG_R9,
    QC_ARM_REG_R10,
    QC_ARM_REG_R11,
    QC_ARM_REG_R12,
    QC_ARM_REG_S0,
    QC_ARM_REG_S1,
    QC_ARM_REG_S2,
    QC_ARM_REG_S3,
    QC_ARM_REG_S4,
    QC_ARM_REG_S5,
    QC_ARM_REG_S6,
    QC_ARM_REG_S7,
    QC_ARM_REG_S8,
    QC_ARM_REG_S9,
    QC_ARM_REG_S10,
    QC_ARM_REG_S11,
    QC_ARM_REG_S12,
    QC_ARM_REG_S13,
    QC_ARM_REG_S14,
    QC_ARM_REG_S15,
    QC_ARM_REG_S16,
    QC_ARM_REG_S17,
    QC_ARM_REG_S18,
    QC_ARM_REG_S19,
    QC_ARM_REG_S20,
    QC_ARM_REG_S21,
    QC_ARM_REG_S22,
    QC_ARM_REG_S23,
    QC_ARM_REG_S24,
    QC_ARM_REG_S25,
    QC_ARM_REG_S26,
    QC_ARM_REG_S27,
    QC_ARM_REG_S28,
    QC_ARM_REG_S29,
    QC_ARM_REG_S30,
    QC_ARM_REG_S31,

    QC_ARM_REG_C1_C0_2,
    QC_ARM_REG_C13_C0_2,
    QC_ARM_REG_C13_C0_3,

    QC_ARM_REG_IPSR,
    QC_ARM_REG_MSP,
    QC_ARM_REG_PSP,
    QC_ARM_REG_CONTROL,
    QC_ARM_REG_IAPSR,
    QC_ARM_REG_EAPSR,
    QC_ARM_REG_XPSR,
    QC_ARM_REG_EPSR,
    QC_ARM_REG_IEPSR,
    QC_ARM_REG_PRIMASK,
    QC_ARM_REG_BASEPRI,
    QC_ARM_REG_BASEPRI_MAX,
    QC_ARM_REG_FAULTMASK,
    QC_ARM_REG_APSR_NZCVQ,
    QC_ARM_REG_APSR_G,
    QC_ARM_REG_APSR_NZCVQG,
    QC_ARM_REG_IAPSR_NZCVQ,
    QC_ARM_REG_IAPSR_G,
    QC_ARM_REG_IAPSR_NZCVQG,
    QC_ARM_REG_EAPSR_NZCVQ,
    QC_ARM_REG_EAPSR_G,
    QC_ARM_REG_EAPSR_NZCVQG,
    QC_ARM_REG_XPSR_NZCVQ,
    QC_ARM_REG_XPSR_G,
    QC_ARM_REG_XPSR_NZCVQG,
    QC_ARM_REG_ENDING, // <-- mark the end of the list or registers

    //> alias registers
    QC_ARM_REG_R13 = QC_ARM_REG_SP,
    QC_ARM_REG_R14 = QC_ARM_REG_LR,
    QC_ARM_REG_R15 = QC_ARM_REG_PC,

    QC_ARM_REG_SB = QC_ARM_REG_R9,
    QC_ARM_REG_SL = QC_ARM_REG_R10,
    QC_ARM_REG_FP = QC_ARM_REG_R11,
    QC_ARM_REG_IP = QC_ARM_REG_R12,
} qc_arm_reg;

#ifdef __cplusplus
}
#endif

#endif
