/* Unicorn Emulator Engine */
/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2015-2017 */
/* This file is released under LGPL2.
   See COPYING.LGPL2 in root directory for more details
*/

#ifndef QNICORN_ARM64_H
#define QNICORN_ARM64_H

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _MSC_VER
#pragma warning(disable : 4201)
#endif

//> ARM64 CPU
typedef enum qc_cpu_aarch64 {
    QC_CPU_AARCH64_A57 = 0,
    QC_CPU_AARCH64_A53,
    QC_CPU_AARCH64_A72,
    QC_CPU_AARCH64_MAX
} qc_cpu_aarch64;

//> ARM64 registers
typedef enum qc_arm64_reg {
    QC_ARM64_REG_INVALID = 0,

    QC_ARM64_REG_X29,
    QC_ARM64_REG_X30,
    QC_ARM64_REG_NZCV,
    QC_ARM64_REG_SP,
    QC_ARM64_REG_WSP,
    QC_ARM64_REG_WZR,
    QC_ARM64_REG_XZR,
    QC_ARM64_REG_B0,
    QC_ARM64_REG_B1,
    QC_ARM64_REG_B2,
    QC_ARM64_REG_B3,
    QC_ARM64_REG_B4,
    QC_ARM64_REG_B5,
    QC_ARM64_REG_B6,
    QC_ARM64_REG_B7,
    QC_ARM64_REG_B8,
    QC_ARM64_REG_B9,
    QC_ARM64_REG_B10,
    QC_ARM64_REG_B11,
    QC_ARM64_REG_B12,
    QC_ARM64_REG_B13,
    QC_ARM64_REG_B14,
    QC_ARM64_REG_B15,
    QC_ARM64_REG_B16,
    QC_ARM64_REG_B17,
    QC_ARM64_REG_B18,
    QC_ARM64_REG_B19,
    QC_ARM64_REG_B20,
    QC_ARM64_REG_B21,
    QC_ARM64_REG_B22,
    QC_ARM64_REG_B23,
    QC_ARM64_REG_B24,
    QC_ARM64_REG_B25,
    QC_ARM64_REG_B26,
    QC_ARM64_REG_B27,
    QC_ARM64_REG_B28,
    QC_ARM64_REG_B29,
    QC_ARM64_REG_B30,
    QC_ARM64_REG_B31,
    QC_ARM64_REG_D0,
    QC_ARM64_REG_D1,
    QC_ARM64_REG_D2,
    QC_ARM64_REG_D3,
    QC_ARM64_REG_D4,
    QC_ARM64_REG_D5,
    QC_ARM64_REG_D6,
    QC_ARM64_REG_D7,
    QC_ARM64_REG_D8,
    QC_ARM64_REG_D9,
    QC_ARM64_REG_D10,
    QC_ARM64_REG_D11,
    QC_ARM64_REG_D12,
    QC_ARM64_REG_D13,
    QC_ARM64_REG_D14,
    QC_ARM64_REG_D15,
    QC_ARM64_REG_D16,
    QC_ARM64_REG_D17,
    QC_ARM64_REG_D18,
    QC_ARM64_REG_D19,
    QC_ARM64_REG_D20,
    QC_ARM64_REG_D21,
    QC_ARM64_REG_D22,
    QC_ARM64_REG_D23,
    QC_ARM64_REG_D24,
    QC_ARM64_REG_D25,
    QC_ARM64_REG_D26,
    QC_ARM64_REG_D27,
    QC_ARM64_REG_D28,
    QC_ARM64_REG_D29,
    QC_ARM64_REG_D30,
    QC_ARM64_REG_D31,
    QC_ARM64_REG_H0,
    QC_ARM64_REG_H1,
    QC_ARM64_REG_H2,
    QC_ARM64_REG_H3,
    QC_ARM64_REG_H4,
    QC_ARM64_REG_H5,
    QC_ARM64_REG_H6,
    QC_ARM64_REG_H7,
    QC_ARM64_REG_H8,
    QC_ARM64_REG_H9,
    QC_ARM64_REG_H10,
    QC_ARM64_REG_H11,
    QC_ARM64_REG_H12,
    QC_ARM64_REG_H13,
    QC_ARM64_REG_H14,
    QC_ARM64_REG_H15,
    QC_ARM64_REG_H16,
    QC_ARM64_REG_H17,
    QC_ARM64_REG_H18,
    QC_ARM64_REG_H19,
    QC_ARM64_REG_H20,
    QC_ARM64_REG_H21,
    QC_ARM64_REG_H22,
    QC_ARM64_REG_H23,
    QC_ARM64_REG_H24,
    QC_ARM64_REG_H25,
    QC_ARM64_REG_H26,
    QC_ARM64_REG_H27,
    QC_ARM64_REG_H28,
    QC_ARM64_REG_H29,
    QC_ARM64_REG_H30,
    QC_ARM64_REG_H31,
    QC_ARM64_REG_Q0,
    QC_ARM64_REG_Q1,
    QC_ARM64_REG_Q2,
    QC_ARM64_REG_Q3,
    QC_ARM64_REG_Q4,
    QC_ARM64_REG_Q5,
    QC_ARM64_REG_Q6,
    QC_ARM64_REG_Q7,
    QC_ARM64_REG_Q8,
    QC_ARM64_REG_Q9,
    QC_ARM64_REG_Q10,
    QC_ARM64_REG_Q11,
    QC_ARM64_REG_Q12,
    QC_ARM64_REG_Q13,
    QC_ARM64_REG_Q14,
    QC_ARM64_REG_Q15,
    QC_ARM64_REG_Q16,
    QC_ARM64_REG_Q17,
    QC_ARM64_REG_Q18,
    QC_ARM64_REG_Q19,
    QC_ARM64_REG_Q20,
    QC_ARM64_REG_Q21,
    QC_ARM64_REG_Q22,
    QC_ARM64_REG_Q23,
    QC_ARM64_REG_Q24,
    QC_ARM64_REG_Q25,
    QC_ARM64_REG_Q26,
    QC_ARM64_REG_Q27,
    QC_ARM64_REG_Q28,
    QC_ARM64_REG_Q29,
    QC_ARM64_REG_Q30,
    QC_ARM64_REG_Q31,
    QC_ARM64_REG_S0,
    QC_ARM64_REG_S1,
    QC_ARM64_REG_S2,
    QC_ARM64_REG_S3,
    QC_ARM64_REG_S4,
    QC_ARM64_REG_S5,
    QC_ARM64_REG_S6,
    QC_ARM64_REG_S7,
    QC_ARM64_REG_S8,
    QC_ARM64_REG_S9,
    QC_ARM64_REG_S10,
    QC_ARM64_REG_S11,
    QC_ARM64_REG_S12,
    QC_ARM64_REG_S13,
    QC_ARM64_REG_S14,
    QC_ARM64_REG_S15,
    QC_ARM64_REG_S16,
    QC_ARM64_REG_S17,
    QC_ARM64_REG_S18,
    QC_ARM64_REG_S19,
    QC_ARM64_REG_S20,
    QC_ARM64_REG_S21,
    QC_ARM64_REG_S22,
    QC_ARM64_REG_S23,
    QC_ARM64_REG_S24,
    QC_ARM64_REG_S25,
    QC_ARM64_REG_S26,
    QC_ARM64_REG_S27,
    QC_ARM64_REG_S28,
    QC_ARM64_REG_S29,
    QC_ARM64_REG_S30,
    QC_ARM64_REG_S31,
    QC_ARM64_REG_W0,
    QC_ARM64_REG_W1,
    QC_ARM64_REG_W2,
    QC_ARM64_REG_W3,
    QC_ARM64_REG_W4,
    QC_ARM64_REG_W5,
    QC_ARM64_REG_W6,
    QC_ARM64_REG_W7,
    QC_ARM64_REG_W8,
    QC_ARM64_REG_W9,
    QC_ARM64_REG_W10,
    QC_ARM64_REG_W11,
    QC_ARM64_REG_W12,
    QC_ARM64_REG_W13,
    QC_ARM64_REG_W14,
    QC_ARM64_REG_W15,
    QC_ARM64_REG_W16,
    QC_ARM64_REG_W17,
    QC_ARM64_REG_W18,
    QC_ARM64_REG_W19,
    QC_ARM64_REG_W20,
    QC_ARM64_REG_W21,
    QC_ARM64_REG_W22,
    QC_ARM64_REG_W23,
    QC_ARM64_REG_W24,
    QC_ARM64_REG_W25,
    QC_ARM64_REG_W26,
    QC_ARM64_REG_W27,
    QC_ARM64_REG_W28,
    QC_ARM64_REG_W29,
    QC_ARM64_REG_W30,
    QC_ARM64_REG_X0,
    QC_ARM64_REG_X1,
    QC_ARM64_REG_X2,
    QC_ARM64_REG_X3,
    QC_ARM64_REG_X4,
    QC_ARM64_REG_X5,
    QC_ARM64_REG_X6,
    QC_ARM64_REG_X7,
    QC_ARM64_REG_X8,
    QC_ARM64_REG_X9,
    QC_ARM64_REG_X10,
    QC_ARM64_REG_X11,
    QC_ARM64_REG_X12,
    QC_ARM64_REG_X13,
    QC_ARM64_REG_X14,
    QC_ARM64_REG_X15,
    QC_ARM64_REG_X16,
    QC_ARM64_REG_X17,
    QC_ARM64_REG_X18,
    QC_ARM64_REG_X19,
    QC_ARM64_REG_X20,
    QC_ARM64_REG_X21,
    QC_ARM64_REG_X22,
    QC_ARM64_REG_X23,
    QC_ARM64_REG_X24,
    QC_ARM64_REG_X25,
    QC_ARM64_REG_X26,
    QC_ARM64_REG_X27,
    QC_ARM64_REG_X28,

    QC_ARM64_REG_V0,
    QC_ARM64_REG_V1,
    QC_ARM64_REG_V2,
    QC_ARM64_REG_V3,
    QC_ARM64_REG_V4,
    QC_ARM64_REG_V5,
    QC_ARM64_REG_V6,
    QC_ARM64_REG_V7,
    QC_ARM64_REG_V8,
    QC_ARM64_REG_V9,
    QC_ARM64_REG_V10,
    QC_ARM64_REG_V11,
    QC_ARM64_REG_V12,
    QC_ARM64_REG_V13,
    QC_ARM64_REG_V14,
    QC_ARM64_REG_V15,
    QC_ARM64_REG_V16,
    QC_ARM64_REG_V17,
    QC_ARM64_REG_V18,
    QC_ARM64_REG_V19,
    QC_ARM64_REG_V20,
    QC_ARM64_REG_V21,
    QC_ARM64_REG_V22,
    QC_ARM64_REG_V23,
    QC_ARM64_REG_V24,
    QC_ARM64_REG_V25,
    QC_ARM64_REG_V26,
    QC_ARM64_REG_V27,
    QC_ARM64_REG_V28,
    QC_ARM64_REG_V29,
    QC_ARM64_REG_V30,
    QC_ARM64_REG_V31,

    //> pseudo registers
    QC_ARM64_REG_PC, // program counter register

    QC_ARM64_REG_CPACR_EL1,

    //> thread registers
    QC_ARM64_REG_TPIDR_EL0,
    QC_ARM64_REG_TPIDRRO_EL0,
    QC_ARM64_REG_TPIDR_EL1,

    QC_ARM64_REG_PSTATE,

    //> exception link registers
    QC_ARM64_REG_ELR_EL0,
    QC_ARM64_REG_ELR_EL1,
    QC_ARM64_REG_ELR_EL2,
    QC_ARM64_REG_ELR_EL3,

    //> stack pointers registers
    QC_ARM64_REG_SP_EL0,
    QC_ARM64_REG_SP_EL1,
    QC_ARM64_REG_SP_EL2,
    QC_ARM64_REG_SP_EL3,

    //> other CP15 registers
    QC_ARM64_REG_TTBR0_EL1,
    QC_ARM64_REG_TTBR1_EL1,

    QC_ARM64_REG_ESR_EL0,
    QC_ARM64_REG_ESR_EL1,
    QC_ARM64_REG_ESR_EL2,
    QC_ARM64_REG_ESR_EL3,

    QC_ARM64_REG_FAR_EL0,
    QC_ARM64_REG_FAR_EL1,
    QC_ARM64_REG_FAR_EL2,
    QC_ARM64_REG_FAR_EL3,

    QC_ARM64_REG_PAR_EL1,

    QC_ARM64_REG_MAIR_EL1,

    QC_ARM64_REG_VBAR_EL0,
    QC_ARM64_REG_VBAR_EL1,
    QC_ARM64_REG_VBAR_EL2,
    QC_ARM64_REG_VBAR_EL3,

    QC_ARM64_REG_ENDING, // <-- mark the end of the list of registers

    //> alias registers

    QC_ARM64_REG_IP0 = QC_ARM64_REG_X16,
    QC_ARM64_REG_IP1 = QC_ARM64_REG_X17,
    QC_ARM64_REG_FP = QC_ARM64_REG_X29,
    QC_ARM64_REG_LR = QC_ARM64_REG_X30,
} qc_arm64_reg;

#ifdef __cplusplus
}
#endif

#endif
