/* Unicorn Emulator Engine */
/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2015-2017 */
/* This file is released under LGPL2.
   See COPYING.LGPL2 in root directory for more details
*/

#ifndef QNICORN_MIPS_H
#define QNICORN_MIPS_H

#ifdef __cplusplus
extern "C" {
#endif

// GCC MIPS toolchain has a default macro called "mips" which breaks
// compilation
#undef mips

#ifdef _MSC_VER
#pragma warning(disable : 4201)
#endif

//> MIPS32 CPUS
typedef enum qc_cpu_mips32 {
    QC_CPU_MIPS32_4KC = 0,
    QC_CPU_MIPS32_4KM,
    QC_CPU_MIPS32_4KECR1,
    QC_CPU_MIPS32_4KEMR1,
    QC_CPU_MIPS32_4KEC,
    QC_CPU_MIPS32_4KEM,
    QC_CPU_MIPS32_24KC,
    QC_CPU_MIPS32_24KEC,
    QC_CPU_MIPS32_24KF,
    QC_CPU_MIPS32_34KF,
    QC_CPU_MIPS32_74KF,
    QC_CPU_MIPS32_M14K,
    QC_CPU_MIPS32_M14KC,
    QC_CPU_MIPS32_P5600,
    QC_CPU_MIPS32_MIPS32R6_GENERIC,
    QC_CPU_MIPS32_I7200,
} qc_cpu_mips32;

//> MIPS64 CPUS
typedef enum qc_cpu_mips64 {
    QC_CPU_MIPS64_R4000 = 0,
    QC_CPU_MIPS64_VR5432,
    QC_CPU_MIPS64_5KC,
    QC_CPU_MIPS64_5KF,
    QC_CPU_MIPS64_20KC,
    QC_CPU_MIPS64_MIPS64R2_GENERIC,
    QC_CPU_MIPS64_5KEC,
    QC_CPU_MIPS64_5KEF,
    QC_CPU_MIPS64_I6400,
    QC_CPU_MIPS64_I6500,
    QC_CPU_MIPS64_LOONGSON_2E,
    QC_CPU_MIPS64_LOONGSON_2F,
    QC_CPU_MIPS64_MIPS64DSPR2
} qc_cpu_mips64;

//> MIPS registers
typedef enum QC_MIPS_REG {
    QC_MIPS_REG_INVALID = 0,
    //> General purpose registers
    QC_MIPS_REG_PC,

    QC_MIPS_REG_0,
    QC_MIPS_REG_1,
    QC_MIPS_REG_2,
    QC_MIPS_REG_3,
    QC_MIPS_REG_4,
    QC_MIPS_REG_5,
    QC_MIPS_REG_6,
    QC_MIPS_REG_7,
    QC_MIPS_REG_8,
    QC_MIPS_REG_9,
    QC_MIPS_REG_10,
    QC_MIPS_REG_11,
    QC_MIPS_REG_12,
    QC_MIPS_REG_13,
    QC_MIPS_REG_14,
    QC_MIPS_REG_15,
    QC_MIPS_REG_16,
    QC_MIPS_REG_17,
    QC_MIPS_REG_18,
    QC_MIPS_REG_19,
    QC_MIPS_REG_20,
    QC_MIPS_REG_21,
    QC_MIPS_REG_22,
    QC_MIPS_REG_23,
    QC_MIPS_REG_24,
    QC_MIPS_REG_25,
    QC_MIPS_REG_26,
    QC_MIPS_REG_27,
    QC_MIPS_REG_28,
    QC_MIPS_REG_29,
    QC_MIPS_REG_30,
    QC_MIPS_REG_31,

    //> DSP registers
    QC_MIPS_REG_DSPCCOND,
    QC_MIPS_REG_DSPCARRY,
    QC_MIPS_REG_DSPEFI,
    QC_MIPS_REG_DSPOUTFLAG,
    QC_MIPS_REG_DSPOUTFLAG16_19,
    QC_MIPS_REG_DSPOUTFLAG20,
    QC_MIPS_REG_DSPOUTFLAG21,
    QC_MIPS_REG_DSPOUTFLAG22,
    QC_MIPS_REG_DSPOUTFLAG23,
    QC_MIPS_REG_DSPPOS,
    QC_MIPS_REG_DSPSCOUNT,

    //> ACC registers
    QC_MIPS_REG_AC0,
    QC_MIPS_REG_AC1,
    QC_MIPS_REG_AC2,
    QC_MIPS_REG_AC3,

    //> COP registers
    QC_MIPS_REG_CC0,
    QC_MIPS_REG_CC1,
    QC_MIPS_REG_CC2,
    QC_MIPS_REG_CC3,
    QC_MIPS_REG_CC4,
    QC_MIPS_REG_CC5,
    QC_MIPS_REG_CC6,
    QC_MIPS_REG_CC7,

    //> FPU registers
    QC_MIPS_REG_F0,
    QC_MIPS_REG_F1,
    QC_MIPS_REG_F2,
    QC_MIPS_REG_F3,
    QC_MIPS_REG_F4,
    QC_MIPS_REG_F5,
    QC_MIPS_REG_F6,
    QC_MIPS_REG_F7,
    QC_MIPS_REG_F8,
    QC_MIPS_REG_F9,
    QC_MIPS_REG_F10,
    QC_MIPS_REG_F11,
    QC_MIPS_REG_F12,
    QC_MIPS_REG_F13,
    QC_MIPS_REG_F14,
    QC_MIPS_REG_F15,
    QC_MIPS_REG_F16,
    QC_MIPS_REG_F17,
    QC_MIPS_REG_F18,
    QC_MIPS_REG_F19,
    QC_MIPS_REG_F20,
    QC_MIPS_REG_F21,
    QC_MIPS_REG_F22,
    QC_MIPS_REG_F23,
    QC_MIPS_REG_F24,
    QC_MIPS_REG_F25,
    QC_MIPS_REG_F26,
    QC_MIPS_REG_F27,
    QC_MIPS_REG_F28,
    QC_MIPS_REG_F29,
    QC_MIPS_REG_F30,
    QC_MIPS_REG_F31,

    QC_MIPS_REG_FCC0,
    QC_MIPS_REG_FCC1,
    QC_MIPS_REG_FCC2,
    QC_MIPS_REG_FCC3,
    QC_MIPS_REG_FCC4,
    QC_MIPS_REG_FCC5,
    QC_MIPS_REG_FCC6,
    QC_MIPS_REG_FCC7,

    //> AFPR128
    QC_MIPS_REG_W0,
    QC_MIPS_REG_W1,
    QC_MIPS_REG_W2,
    QC_MIPS_REG_W3,
    QC_MIPS_REG_W4,
    QC_MIPS_REG_W5,
    QC_MIPS_REG_W6,
    QC_MIPS_REG_W7,
    QC_MIPS_REG_W8,
    QC_MIPS_REG_W9,
    QC_MIPS_REG_W10,
    QC_MIPS_REG_W11,
    QC_MIPS_REG_W12,
    QC_MIPS_REG_W13,
    QC_MIPS_REG_W14,
    QC_MIPS_REG_W15,
    QC_MIPS_REG_W16,
    QC_MIPS_REG_W17,
    QC_MIPS_REG_W18,
    QC_MIPS_REG_W19,
    QC_MIPS_REG_W20,
    QC_MIPS_REG_W21,
    QC_MIPS_REG_W22,
    QC_MIPS_REG_W23,
    QC_MIPS_REG_W24,
    QC_MIPS_REG_W25,
    QC_MIPS_REG_W26,
    QC_MIPS_REG_W27,
    QC_MIPS_REG_W28,
    QC_MIPS_REG_W29,
    QC_MIPS_REG_W30,
    QC_MIPS_REG_W31,

    QC_MIPS_REG_HI,
    QC_MIPS_REG_LO,

    QC_MIPS_REG_P0,
    QC_MIPS_REG_P1,
    QC_MIPS_REG_P2,

    QC_MIPS_REG_MPL0,
    QC_MIPS_REG_MPL1,
    QC_MIPS_REG_MPL2,

    QC_MIPS_REG_CP0_CONFIG3,
    QC_MIPS_REG_CP0_USERLOCAL,
    QC_MIPS_REG_CP0_STATUS,

    QC_MIPS_REG_ENDING, // <-- mark the end of the list or registers

    // alias registers
    QC_MIPS_REG_ZERO = QC_MIPS_REG_0,
    QC_MIPS_REG_AT = QC_MIPS_REG_1,
    QC_MIPS_REG_V0 = QC_MIPS_REG_2,
    QC_MIPS_REG_V1 = QC_MIPS_REG_3,
    QC_MIPS_REG_A0 = QC_MIPS_REG_4,
    QC_MIPS_REG_A1 = QC_MIPS_REG_5,
    QC_MIPS_REG_A2 = QC_MIPS_REG_6,
    QC_MIPS_REG_A3 = QC_MIPS_REG_7,
    QC_MIPS_REG_T0 = QC_MIPS_REG_8,
    QC_MIPS_REG_T1 = QC_MIPS_REG_9,
    QC_MIPS_REG_T2 = QC_MIPS_REG_10,
    QC_MIPS_REG_T3 = QC_MIPS_REG_11,
    QC_MIPS_REG_T4 = QC_MIPS_REG_12,
    QC_MIPS_REG_T5 = QC_MIPS_REG_13,
    QC_MIPS_REG_T6 = QC_MIPS_REG_14,
    QC_MIPS_REG_T7 = QC_MIPS_REG_15,
    QC_MIPS_REG_S0 = QC_MIPS_REG_16,
    QC_MIPS_REG_S1 = QC_MIPS_REG_17,
    QC_MIPS_REG_S2 = QC_MIPS_REG_18,
    QC_MIPS_REG_S3 = QC_MIPS_REG_19,
    QC_MIPS_REG_S4 = QC_MIPS_REG_20,
    QC_MIPS_REG_S5 = QC_MIPS_REG_21,
    QC_MIPS_REG_S6 = QC_MIPS_REG_22,
    QC_MIPS_REG_S7 = QC_MIPS_REG_23,
    QC_MIPS_REG_T8 = QC_MIPS_REG_24,
    QC_MIPS_REG_T9 = QC_MIPS_REG_25,
    QC_MIPS_REG_K0 = QC_MIPS_REG_26,
    QC_MIPS_REG_K1 = QC_MIPS_REG_27,
    QC_MIPS_REG_GP = QC_MIPS_REG_28,
    QC_MIPS_REG_SP = QC_MIPS_REG_29,
    QC_MIPS_REG_FP = QC_MIPS_REG_30,
    QC_MIPS_REG_S8 = QC_MIPS_REG_30,
    QC_MIPS_REG_RA = QC_MIPS_REG_31,

    QC_MIPS_REG_HI0 = QC_MIPS_REG_AC0,
    QC_MIPS_REG_HI1 = QC_MIPS_REG_AC1,
    QC_MIPS_REG_HI2 = QC_MIPS_REG_AC2,
    QC_MIPS_REG_HI3 = QC_MIPS_REG_AC3,

    QC_MIPS_REG_LO0 = QC_MIPS_REG_HI0,
    QC_MIPS_REG_LO1 = QC_MIPS_REG_HI1,
    QC_MIPS_REG_LO2 = QC_MIPS_REG_HI2,
    QC_MIPS_REG_LO3 = QC_MIPS_REG_HI3,
} QC_MIPS_REG;

#ifdef __cplusplus
}
#endif

#endif
